@@ -1,5 +1,9 @@
+from __future__ import annotations
+
 import json
-from typing import List, Tuple
+import logging
+import sys
+from typing import Union
 
 import numpy as np
 import torch
@@ -13,8 +17,11 @@ from torchtree.cli.evolution import (
     create_site_model_srd06_mus,
     create_taxa,
 )
+from torchtree.cli.jacobians import create_jacobians
 from torchtree.distributions import Distribution
 
+logger = logging.getLogger(__name__)
+
 
 def create_variational_parser(subprasers):
     parser = subprasers.add_parser(
@@ -37,8 +44,7 @@ def create_variational_parser(subprasers):
         help="""variational distribution family""",
     )
     parser.add_argument(
-        '-e',
-        '--eta',
+        '--lr',
         default=0.1,
         type=float,
         help="""learning rate (default: 0.1)""",
@@ -58,6 +64,7 @@ def create_variational_parser(subprasers):
     parser.add_argument(
         '--K_grad_samples',
         type=int,
+        default=1,
         help="number of samples for Monte Carlo estimate of gradients"
         " using multisample objective",
     )
@@ -97,6 +104,12 @@ def create_variational_parser(subprasers):
         required=False,
         help="""checkpoint file from a meanfield analysis""",
     )
+    parser.add_argument(
+        '--convergence_every',
+        type=int,
+        default=100,
+        help="""convergence every N iterations""",
+    )
     parser.set_defaults(func=build_advi)
     return parser
 
@@ -140,8 +153,8 @@ def create_fullrank_from_meanfield(params, path):
         elif '.scale.unres' in param['id']:
             log_scales.append(param)
         else:
-            print(param['id'])
-            exit(2)
+            sys.stderr.write(param['id'])
+            sys.exit(2)
     sorted(
         locs,
         key=lambda x: params.index(x['id'].replace('.loc', '').replace('var.', '')),
@@ -242,6 +255,45 @@ def create_flexible_variational(arg, json_object):
     return joint_var, var_parameters
 
 
+def create_realnvp(var_id, json_object, arg):
+    group_map = {}
+    parameters = {'UNSPECIFIED': []}
+    gather_parameters(json_object, group_map, parameters)
+    res = apply_transforms_for_fullrank(var_id, parameters['UNSPECIFIED'])
+    x = list(map(lambda x: x[1], res))
+
+    params = torch.cat(list(map(lambda x: torch.tensor(x[2]), res)))
+
+    distr = {
+        "id": var_id,
+        "type": "RealNVP",
+        "x": x,
+        "base": {
+            "id": "base",
+            "type": "Distribution",
+            "distribution": "torchtree.distributions.Normal",
+            "x": {"id": "dummy", "type": "Parameter", "zeros": params.shape[-1]},
+            "parameters": {
+                "loc": {
+                    "id": f"{var_id}.base.loc",
+                    "type": "Parameter",
+                    "zeros": params.shape[-1],
+                },
+                "scale": {
+                    "id": f"{var_id}.base.scale",
+                    "type": "Parameter",
+                    "ones": params.shape[-1],
+                },
+            },
+        },
+        "n_blocks": 2,
+        "hidden_size": 2,
+        "n_hidden": 1,
+    }
+    var_parameters = (var_id,)
+    return distr, var_parameters
+
+
 def gather_parameters(json_object: dict, group_map: dict, parameters: dict):
     if isinstance(json_object, list):
         for element in json_object:
@@ -294,9 +346,13 @@ def apply_sigmoid_transformed(json_object, value=None):
         del json_object['tensor']
         del json_object['full']
     else:
-        print('error from apply_sigmoid_transformed')
-        print(json_object)
-        exit(1)
+        logger.debug(
+            'apply_sigmoid_transformed only works on json object containing'
+            ' tensor or full'
+        )
+        sys.stderr.write('error from apply_sigmoid_transformed\n')
+        sys.stderr.write(json_object)
+        sys.exit(1)
     return unres_id
 
 
@@ -449,7 +505,7 @@ def create_weibull_distribution(var_id, x_unres, json_object, scale, concentrati
 
 def create_meanfield(
     var_id: str, json_object: dict, distribution: str
-) -> Tuple[List[str], List[str]]:
+) -> tuple[list[str], list[str]]:
     distributions = []
     var_parameters = []
     parameters = []
@@ -538,8 +594,8 @@ def create_meanfield(
 
 def apply_transforms_for_fullrank(
     var_id: str,
-    json_object: [dict, list],
-) -> List[Tuple[str, str, list]]:
+    json_object: Union[dict, list],
+) -> list[tuple[str, str, list]]:
     var_parameters = []
     if isinstance(json_object, list):
         for element in json_object:
@@ -599,9 +655,9 @@ def apply_transforms_for_fullrank(
     return var_parameters
 
 
-def create_variational_model(id_, joint, arg) -> Tuple[dict, List[str]]:
+def create_variational_model(id_, joint, arg) -> tuple[dict, list[str]]:
     variational = {'id': id_, 'type': 'JointDistributionModel'}
-    if arg.variational == 'meanfield':
+    if len(arg.variational) == 1 and arg.variational[0] == 'meanfield':
         distributions, parameters = create_meanfield(id_, joint, arg.distribution)
     elif (
         arg.variational == 'fullrank'
@@ -609,6 +665,8 @@ def create_variational_model(id_, joint, arg) -> Tuple[dict, List[str]]:
         and arg.variational[0] == 'fullrank'
     ):
         return create_fullrank(id_, joint, arg)
+    elif len(arg.variational) == 1 and arg.variational[0] == 'realnvp':
+        return create_realnvp(id_, joint, arg.distribution)
     else:
         distributions, parameters = create_flexible_variational(arg, joint)
     variational['distributions'] = distributions
@@ -616,7 +674,7 @@ def create_variational_model(id_, joint, arg) -> Tuple[dict, List[str]]:
 
 
 def create_advi(joint, variational, parameters, arg):
-    if arg.K_grad_samples is not None:
+    if arg.K_grad_samples > 1:
         grad_samples = [arg.grad_samples, arg.K_grad_samples]
     else:
         grad_samples = arg.grad_samples
@@ -629,8 +687,8 @@ def create_advi(joint, variational, parameters, arg):
         'id': 'advi',
         'type': 'Optimizer',
         'algorithm': 'torch.optim.Adam',
+        'options': {'lr': arg.lr},
         'maximize': True,
-        'lr': arg.eta,
         'checkpoint': checkpoint,
         'iterations': arg.iter,
         'loss': {
@@ -647,7 +705,7 @@ def create_advi(joint, variational, parameters, arg):
         'type': 'StanVariationalConvergence',
         'max_iterations': arg.iter,
         'loss': 'elbo',
-        'every': 100,
+        'every': arg.convergence_every,
         'samples': arg.elbo_samples,
         'tol_rel_obj': arg.tol_rel_obj,
     }
@@ -662,8 +720,12 @@ def create_advi(joint, variational, parameters, arg):
 def create_sampler(id_, var_id, parameters, arg):
     if arg.stem:
         file_name = arg.stem + '-samples.csv'
+        tree_file_name = arg.stem + '-samples.trees'
     else:
         file_name = 'samples.csv'
+        tree_file_name = 'samples.trees'
+
+    parameters2 = list(filter(lambda x: 'tree.ratios' != x, parameters))
 
     return {
         "id": id_,
@@ -675,30 +737,19 @@ def create_sampler(id_, var_id, parameters, arg):
                 "id": "logger",
                 "type": "Logger",
                 "file_name": file_name,
-                "parameters": parameters,
+                "parameters": ['joint', 'like', var_id] + parameters2,
                 "delimiter": "\t",
-            }
+            },
+            {
+                "id": "tree.logger",
+                "type": "TreeLogger",
+                "file_name": tree_file_name,
+                "tree_model": "tree",
+            },
         ],
     }
 
 
-def create_jacobians(json_object):
-    params = []
-    if isinstance(json_object, list):
-        for element in json_object:
-            params.extend(create_jacobians(element))
-    elif isinstance(json_object, dict):
-        if 'type' in json_object and json_object['type'] == 'TransformedParameter':
-            if not (
-                json_object['transform'] == 'torch.distributions.AffineTransform'
-                and json_object['parameters']['scale'] == 1.0
-            ):
-                params.append(json_object['id'])
-        for value in json_object.values():
-            params.extend(create_jacobians(value))
-    return params
-
-
 def build_advi(arg):
     json_list = []
     taxa = create_taxa('taxa', arg)
@@ -719,7 +770,7 @@ def build_advi(arg):
 
     # convert Parameters with constraints to TransformedParameters
     # and create variational distribution
-    var_dic, var_parameters = create_variational_model('var', json_list, arg)
+    var_dic, var_parameters = create_variational_model('variational', json_list, arg)
 
     # extract references of TransformedParameters but not those coming from the
     # variational distribution
@@ -732,7 +783,7 @@ def build_advi(arg):
 
     json_list.append(var_dic)
 
-    advi_dic = create_advi('joint', 'var', var_parameters, arg)
+    advi_dic = create_advi('joint', 'variational', var_parameters, arg)
     json_list.append(advi_dic)
 
     parameters = []
@@ -764,6 +815,8 @@ def build_advi(arg):
             parameters.append('gmrf.precision')
         elif arg.coalescent == 'exponential':
             parameters.append('coalescent.growth')
+        elif arg.coalescent == 'piecewise':
+            parameters.append('coalescent.growth')
     elif arg.birth_death is not None:
         parameters.append("bdsk.R")
         parameters.append("bdsk.delta")
@@ -793,5 +846,5 @@ def build_advi(arg):
             parameters.append("sitemodel.shape")
 
     if arg.samples > 0:
-        json_list.append(create_sampler('sampler', 'var', parameters, arg))
+        json_list.append(create_sampler('sampler', 'variational', parameters, arg))
     return json_list
