@@ -2,6 +2,7 @@ import argparse
 import json
 
 from torchtree.cli.advi import create_variational_parser
+from torchtree.cli.hmc import create_hmc_parser
 from torchtree.cli.map import create_map_parser
 
 
@@ -17,6 +18,8 @@ def main():
 
     create_map_parser(subprasers)
 
+    create_hmc_parser(subprasers)
+
     arg = parser.parse_args()
     json_dic = arg.func(arg)
 
