@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 import abc
 from abc import ABC
 from io import StringIO
-from typing import List, Optional, Union
+from typing import Optional, Union
 
 import torch
 from dendropy import TaxonNamespace, Tree
@@ -157,12 +159,12 @@ class TreeModel(Model):
 
     @property
     @abc.abstractmethod
-    def postorder(self) -> List[List[int]]:
+    def postorder(self) -> list[list[int]]:
         ...
 
     @property
     @abc.abstractmethod
-    def taxa(self) -> List[str]:
+    def taxa(self) -> list[str]:
         ...
 
     @abc.abstractmethod
@@ -206,28 +208,32 @@ class AbstractTreeModel(TreeModel, ABC):
         self.write_newick(out, **kwargs)
         return out.getvalue()
 
-    def write_newick(self, steam, **kwargs) -> None:
-        self._write_newick(self.tree.seed_node, steam, **kwargs)
+    def write_newick(self, stream, **kwargs) -> None:
+        self._write_newick(self.tree.seed_node, stream, **kwargs)
 
-    def _write_newick(self, node, steam, **kwargs) -> None:
+    def _write_newick(self, node, stream, **kwargs) -> None:
         if not node.is_leaf():
-            steam.write('(')
+            stream.write('(')
             for i, child in enumerate(node.child_node_iter()):
-                self._write_newick(child, steam, **kwargs)
+                self._write_newick(child, stream, **kwargs)
                 if i == 0:
-                    steam.write(',')
-            steam.write(')')
+                    stream.write(',')
+            stream.write(')')
         else:
             taxon_index = kwargs.get('taxon_index', None)
             if not taxon_index:
-                steam.write(str(node.taxon).strip("'"))
+                stream.write(str(node.taxon).strip("'"))
             else:
-                steam.write(str(node.index + 1))
+                stream.write(str(node.index + 1))
         if node.parent_node is not None:
             branch_lengths = kwargs.get('branch_lengths', self.branch_lengths())
-            steam.write(':{}'.format(branch_lengths[node.index]))
+            # unrooted trees have 2N-3 branches but it is writing a binary tree
+            if node.index == len(branch_lengths):
+                stream.write(':0')
+            else:
+                stream.write(':{}'.format(branch_lengths[node.index]))
         else:
-            steam.write(';')
+            stream.write(';')
 
 
 @register_class
@@ -254,7 +260,7 @@ class UnRootedTreeModel(AbstractTreeModel):
         newick: str,
         branch_lengths: Union[dict, list, str],
         taxa: Union[dict, list, str],
-        **kwargs
+        **kwargs,
     ):
         r"""
         Factory for creating tree models in JSON format.
@@ -274,7 +280,7 @@ class UnRootedTreeModel(AbstractTreeModel):
 
         tree_model = {
             'id': id_,
-            'type': 'torchtree.evolution.tree_model.UnRootedTreeModel',
+            'type': 'UnRootedTreeModel',
             'newick': newick,
         }
         if 'keep_branch_lengths' in kwargs and kwargs['keep_branch_lengths']:
@@ -438,7 +444,7 @@ class TimeTreeModel(AbstractTreeModel):
         newick: str,
         internal_heights: Union[dict, list, str],
         taxa: Union[dict, list, str],
-        **kwargs
+        **kwargs,
     ):
         r"""
         Factory for creating tree models in JSON format.
@@ -458,7 +464,7 @@ class TimeTreeModel(AbstractTreeModel):
 
         tree_model = {
             'id': id_,
-            'type': 'torchtree.evolution.tree_model.TimeTreeModel',
+            'type': 'TimeTreeModel',
             'newick': newick,
         }
         if 'keep_branch_lengths' in kwargs and kwargs['keep_branch_lengths']:
@@ -545,6 +551,20 @@ class ReparameterizedTimeTreeModel(TimeTreeModel, CallableModel):
             self.heights_need_update = False
         return self._node_heights
 
+    def handle_model_changed(self, model, obj, index) -> None:
+        self.lp_needs_update = True
+        self.branch_lengths_need_update = True
+        self.heights_need_update = True
+        self.fire_model_changed(self)
+
+    def handle_parameter_changed(
+        self, variable: AbstractParameter, index, event
+    ) -> None:
+        self.lp_needs_update = True
+        self.branch_lengths_need_update = True
+        self.heights_need_update = True
+        self.fire_model_changed(self)
+
     def _call(self, *args, **kwargs) -> torch.Tensor:
         if self.heights_need_update:
             self.update_node_heights()
@@ -567,7 +587,7 @@ class ReparameterizedTimeTreeModel(TimeTreeModel, CallableModel):
         ratios: Union[dict, list, str],
         root_height: Union[dict, list, str],
         taxa: Union[dict, list, str],
-        **kwargs
+        **kwargs,
     ):
         r"""
         Factory for creating tree models in JSON format.
@@ -587,7 +607,7 @@ class ReparameterizedTimeTreeModel(TimeTreeModel, CallableModel):
 
         tree_model = {
             'id': id_,
-            'type': 'torchtree.evolution.tree_model.ReparameterizedTimeTreeModel',
+            'type': 'ReparameterizedTimeTreeModel',
             'newick': newick,
         }
         if 'keep_branch_lengths' in kwargs and kwargs['keep_branch_lengths']:
